Mule Salesforce Composite Connector Demo
========================================

Mule Studio demo for showing how to take advantage of DataSense, gather sub-requests and execute them in one batch.

It creates four sub-requests, executes, parses the results and logs them into a file. 

The sub-requests are: update account, update contact, retrieve account, retrieve contact.

## Prerequisites

* Java 7 
* Anypoint Studio 5.4.x
* Mule Runtime 3.7.0 EE
* DataWeave
* Access to a Salesforce instance

## Import the demo project into Anypoint Studio

* Go to **File > Import**
* Select **Anypoint Studio Project from External Location** (under the parent folder "Anypoint Studio")

![Import Menu](images/import-project.png)

* Provide the root path to the demo project folder.
* Select Mule Runtime (3.7.x EE).

![File Selection](images/import-project-1.png)

* Click **Finish**.  
* Set Salesforce credentials inside the file `src/main/app/mule-app.properties`.


	
	config.username=<USERNAME> user name within Salesforce system

	config.password=<PASSWORD> password within Salesforce system

	config.consumerKey=<CONSUMER_KEY> consumer key of the connected app through which you are accessing salesforce
 
	config.consumerSecret=<CONSUMER_SECRET> consumer secret of the connected app through which you are accessing salesforce

	config.securityToken=<SECURITY_TOKEN> security token within Salesforce system

	config.tokenEndpoint=<ENDPOINT_REST_URL> e.g https://login.salesforce.com/services/oauth2/token [url to server responsible for authentication]
	

How To Run the Demo
-------------------
* Run the demo application by selecting **Run as Mule Application** 
* Open a browser and access the URL `http://localhost:8081`. You should see the demo application deployed:

A html form with two sections will show up which allows to access the selection menu of the demo.

1. Prerequisite create account form 

	![Demo Index](images/demo-prerequisite-create-account.png)

2. Prerequisite create contact form 

	![Demo Index](images/demo-prerequisite-create-contact.png)
	
3. Batch operation form 

	![Demo Index](images/demo-batch.png)


About the Demo
--------------


The project provides three forms, two with the prerequisite data and one containing the information necessary for the batch operation. 

### 1. Prerequisite screens
  
Will be used to create one account and one contact necessary for the batch operation.

**1.1 Create Account**

 This section contains a field "Account Name" - used by the create account operation sub-request
 
 The sub-request will create a new account with the provided name.
  
	When you hit "Execute create account" the flow is triggered, sub-requests are generated based on data provided by you and executed against Salesforce.
	The result of execution is written into '/src/main/resources/output/account_output.json' in json format. Each result of sub-request is written in this format:
	{
		<what_subrequest_represents> : <result_of_subrequest>
	}


**1.2 Create Contact**

 This section contains a field "Contact Name" - used by the create contact operation sub-request
 
 The sub-request will create a new contact with the provided name.
  
	When you hit "Execute create contact" the flow is triggered, sub-requests are generated based on data provided by you and executed against Salesforce.
	The result of execution is written into '/src/main/resources/output/contact_output.json' in json format. Each result of sub-request is written in this format:
	{
		<what_subrequest_represents> : <result_of_subrequest>
	}

### 2. Batch request screen
   
Contains four data sections for the following operations:

**2.1 Update Account**

This section contains the following fields that are going to be used by sub-request which is updating an existing account:

* "Account Id" - id of the account that you are going to update
* "Phone" - new phone of the account after updating it

**2.2 Update Contact**

This section contains the following fields that are going to be used by sub-request which is updating an existing contact:

* "Contact Id" - id of the account that you are going to update
* "Title" - new title of the contact after updating it

**2.3 Retrieve Account**

This section contains a field "Account Id" -used by the retrieve account operation 

The sub-request will retrieve account info("Id", "Name" and "Phone")

**2.4 Retrieve Contact**

This section contains a field "Contact Id" -used by the retrieve contact operation 

The sub-request will retrieve account info("Id", "Name" and "Title")

	
	When you change the id of contact and account within "Update Account sub-request data" and "Update Contact sub-request data" the ids will be populated into "Account Id" of section "Retrieve Account sub-request data" and "Contact Id" of section "Retrieve Contact sub-request data", respectively.
	When you hit "Execute sub-requests" the flow is triggered, sub-requests are generated based on data provided by you and executed against Salesforce.
	The results of execution are written into /src/main/resources/output/batch_output.json in json format. Each result of sub-request is written in this format:
	{
	<what_subrequest_represents> : <result_of_subrequest>
	}

## About the flows


### Prerequisite flows

#### 1. Create account flow

**1.1 create-accounts-input-data-parser-and-processor-triggerer-flow**:	contains an "HTTP" endpoint binded to "/execute-create-accounts" and is responsible of formatting input data that comes with the request and writes it into "src/main/resources/input/account/account.xml" in xml format (this is a proprietary format used only for this demo).

![Demo Index](images/create-accounts-input-data-parser-and-processor-triggerer-flow.png)

**1.2 create-accounts-preparator-flow**:contains an "File" endpoint that is monitoring "src/main/resources/input/account" directory and when "accounts.xml" is written by previous flow, parses it and splits it into items, each item is routed to "Pre" operation responsible for preparing sub-request based on what data represent.
 
The collection of prepared sub-requests is passed to a queue binded to path "/createAccountToBeExecuted".

![Demo Index](images/create-accounts-preparator-flow.png)

**1.3 create-account-requests-executor-flow**:contains an "VM" (queue) endpoint binded to "/createAccountToBeExecuted"" and sends sub-requests that came through "VM" to Salesforce via "Execute Create Account" operation of "Salesforce Composite" connector. 

The collection of result of execution is splitted and routed through a "First Successful" component that matches the parser operation with the result that is correlated to it. 

The result collection of parsing is then logged into "src/main/resources/output/account_output.json" in json format.

![Demo Index](images/create-account-requests-executor-flow.png)

#### 2. Create contact flow

**2.1 create-contacts-input-data-parser-and-processor-triggerer-flow**:	contains an "HTTP" endpoint binded to "/execute-create-contacts" and is responsible of formatting input data that comes with the request and writes it into "src/main/resources/input/contact/contact.xml" in xml format (this is a proprietary format used only for this demo).

![Demo Index](images/create-contacts-input-data-parser-and-processor-triggerer-flow.png)

**2.2 create-contacts-preparator-flow**:contains an "File" endpoint that is monitoring "src/main/resources/input/contact" directory and when "contact.xml" is written by previous flow, parses it and splits it into items, each item is routed to "Pre" operation responsible for preparing sub-request based on what data represent.
 
The collection of prepared sub-requests is passed to a queue binded to path "/createContactToBeExecuted".

![Demo Index](images/create-contacts-preparator-flow.png)

**2.3 create-contact-requests-executor-flow**:contains an "VM" (queue) endpoint binded to "/createContactToBeExecuted"" and sends sub-requests that came through "VM" to Salesforce via "Execute Create Contact" operation of "Salesforce Composite" connector. 

The collection of result of execution is splitted and routed through a "First Successful" component that matches the parser operation with the result that is correlated to it. 

The result collection of parsing is then logged into "src/main/resources/output/contact_output.json" in json format.

![Demo Index](images/create-contact-requests-executor-flow.png)


###3. Batch flows

There are four flows, each with a different purpose, as described below:

**3.1 input-data-form-provider-flow**:contains an "HTTP" endpoint binded to "/" and is responsible of parsing "src/main/resources/form.html" and returning it.

It contains a html form having fields for colllecting data needed for preparing sub-requests. The data are submitted to "/execute-subrequests" via ajax call when user clicks on "Execute sub-requests"  

![Demo Index](images/input-data-form-provider-flow.png)

**3.2 input-data-parser-and-processor-triggerer-flow** contains an "HTTP" endpoint binded to "/execute-subrequests" and is responsible of formatting input data that comes with the request and writes it into "src/main/resources/input/items.xml" in xml format (this is a proprietary format used only for this demo).

![Demo Index](images/input-data-parser-and-processor-triggerer-flow.png)

**3.3 sub-requests-preparator-flow**: contains an "File" endpoint that is monitoring "src/main/resources/input" directory and when "items.xml" is written by previous flow, parses it and splits it into items, each item is routed to "Pre" operation responsible for preparing sub-request based on what data represent.

The collection of prepared sub-requests is passed to a queue binded to path "/toBeExecuted". 

![Demo Index](images/sub-requests-preparator-flow.png)

**3.4 sub-requests-executor-flow**: contains an "VM" (queue) endpoint binded to "/toBeExecuted" and sends sub-requests that came through "VM" to Salesforce via "Execute Composite Batch" operation of "Salesforce Composite" connector.

The collection of result of execution is splitted and routed through a "First Successful" component that matches the parser operation with the result that is correlated to it.

The result collection of parsing is then logged into "src/main/resources/output/batch_output.json" in json format.

	For sub-requests results that ended with errors or for which there was no operation to correlate with, the message that is logged into "src/main/resources/output/batch_output.json" is going to have the following format:
	{
		message: "Unable to parse sub-request result",
		subRequestCorrelationId: <sub_request_correlation_id>,
		rawResponse: <raw_result_from_server>
	}

![Demo Index](images/sub-requests-executor-flow.png)

### 4. Get Responses flows

**4.1 get-create-account-reponse**: contains an "HTTP" endpoint binded to "/get-create-account-reponse" and is responsible of reading "/src/main/resources/output/account_output.json" and returning it to the UI- 'form.html'.

![Demo Index](images/get-create-account-reponse.png)

**4.2 get-create-contact-reponse**: contains an "HTTP" endpoint binded to "/get-create-contact-reponse" and is responsible of reading "/src/main/resources/output/contact_output.json" and returning it to the UI- 'form.html'.

![Demo Index](images/get-create-contact-reponse.png)


**4.3 get-batch-response**: contains an "HTTP" endpoint binded to "/get-batch-response" and is responsible of reading "/src/main/resources/output/batch_output.json" and returning it to the UI- 'form.html'.

![Demo Index](images/get-create-account-reponse.png)

**4.4 get-loader-image**: contains an "HTTP" endpoint binded to "/get-loader-image" and is responsible of returning a gif loader resource.

![Demo Index](images/get-create-account-reponse.png)	
	